(function() {
var GridViewContextMenuItemType = {
 FullExpand: "FullExpand",
 FullCollapse: "FullCollapse",
 SortAscending: "SortAscending",
 SortDescending: "SortDescending",
 ClearSorting: "ClearSorting",
 ShowFilterBuilder: "ShowFilterEditor",
 ShowFilterRow: "ShowFilterRow",
 ClearFilter: "ClearFilter",
 ShowFilterRowMenu: "ShowFilterRowMenu",
 GroupByColumn: "GroupByColumn",
 UngroupColumn: "UngroupColumn",
 ClearGrouping: "ClearGrouping",
 ShowGroupPanel: "ShowGroupPanel",
 ShowSearchPanel: "ShowSearchPanel",
 ShowColumn: "ShowColumn",
 HideColumn: "HideColumn",
 ShowCustomizationDialog: "ShowCustomizationDialog",
 ShowCustomizationWindow: "ShowCustomizationWindow",
 ShowFooter: "ShowFooter",
 ExpandRow: "ExpandRow",
 CollapseRow: "CollapseRow",
 ExpandDetailRow: "ExpandDetailRow",
 CollapseDetailRow: "CollapseDetailRow",
 NewRow: "NewRow",
 EditRow: "EditRow",
 DeleteRow: "DeleteRow",
 Refresh: "Refresh",
 SummarySum: "SummarySum",
 SummaryMin: "SummaryMin",
 SummaryMax: "SummaryMax",
 SummaryAverage: "SummaryAverage",
 SummaryCount: "SummaryCount",
 SummaryNone: "SummaryNone",
 GroupSummarySum: "GroupSummarySum",
 GroupSummaryMin: "GroupSummaryMin",
 GroupSummaryMax: "GroupSummaryMax",
 GroupSummaryAverage: "GroupSummaryAverage",
 GroupSummaryCount: "GroupSummaryCount",
 GroupSummaryNone: "GroupSummaryNone",
 CustomItem: "CustomItem"
};
var CONTEXT_MENU_CLIENT_STATE_KEY = "contextMenu";
var GridViewContextMenuHelper = ASPx.CreateClass(ASPx.GridCommandMenuHelper, {
 constructor: function(grid) {
  this.constructor.prototype.constructor.call(this, ASPx.ASPxClientGridExportFormat);
  this.grid = grid;
  this.emptyElementIndex = -1;
  this.sourceContextMenuRow = null;
  this.activeContextMenu = null;
  this.contextMenuActivating = false;
  this.clickedMenuItem = null;
 },
 GetGridName: function() { return this.grid.name; },
 GetCustWindowSuffix: function() { return this.grid.CustomizationWindowSuffix; }, 
 IsDetailGrid: function() { return this.grid.IsDetailGrid(); },
 GetGroupPanelContextMenu: function() { return this.grid.GetGroupPanelContextMenu(); },
 GetColumnContextMenu: function() { return this.grid.GetColumnContextMenu(); },
 GetRowContextMenu: function() { return this.grid.GetRowContextMenu(); },
 GetFooterContextMenu: function() { return this.grid.GetFooterContextMenu(); },
 GetGroupFooterContextMenu: function() { return this.grid.GetGroupFooterContextMenu(); },
 GetRowHotTrackStyle: function() { return this.rowHotTrackStyle; },
 GetEnableAccessibility: function() { return this.grid.accessibilityCompliant; },
 IsCustWindowVisible: function() { return this.grid.IsCustomizationWindowVisible(); },
 HasContextMenuEventHandler: function() { return !this.grid.ContextMenu.IsEmpty(); },
 RaiseContextMenu: function(objectType, index, htmlEvent, menu, showBrowserMenu) { 
  return this.grid.RaiseContextMenu(objectType, index, htmlEvent, menu, showBrowserMenu);
 },
 RaiseContextMenuItemClick: function(e, itemInfo) {
  return this.grid.RaiseContextMenuItemClick(e, itemInfo);
 },
 HasAnyContextMenu: function() {
  return this.GetGroupPanelContextMenu() || this.GetColumnContextMenu() || this.GetRowContextMenu() || this.GetFooterContextMenu();
 },
 SendContextMenuCallback: function(args) {
  return this.grid.gridCallBack([ ASPxClientGridViewCallbackCommand.ContextMenu ].concat(args));
 },
 SendContextMenuPostBack: function(args) {
  return this.grid.gridPostBack([ ASPxClientGridViewCallbackCommand.ContextMenu ].concat(args));
 },
 OnAfterCallack: function() { 
  this.ResetState();
 },
 GetCallbackState: function() {
  if(!this.clickedMenuItem)
   return "";
  var menu = this.clickedMenuItem.menu;
  var elementInfo = menu.elementInfo;
  return menu.cpType + "," + this.clickedMenuItem.indexPath + "," + elementInfo.index;
 },
 SaveClientState: function() {
  this.SetClientStateValue(this.GetCallbackState());
 },
 ResetState: function() {
  this.clickedMenuItem = null;
  this.SetClientStateValue(null);
 },
 SetClientStateValue: function(value) {
  this.grid.stateObject[CONTEXT_MENU_CLIENT_STATE_KEY] = value;
 },
 OnContextMenuClick: function(e) {
  var showDefaultMenu = ASPx.EventStorage.getInstance().Load(e);
  if(showDefaultMenu)
   return showDefaultMenu;
  if(this.IsDetailGrid())
   ASPx.EventStorage.getInstance().Save(e, true);
  var args = this.GetContextMenuArgs(e);
  if(!args.objectType && !this.HasAnyContextMenu())
   return true;
  var menu = this.GetPreparedContextMenu(args);
  var showBrowserMenu = !menu && !this.HasContextMenuEventHandler();
  showBrowserMenu = this.RaiseContextMenu(args.objectType, args.index, e, menu, showBrowserMenu);
  if(menu && !showBrowserMenu) {
   if(this.GetEnableAccessibility())
    menu.accessibleFocusElement = ASPx.Evt.GetEventSource(e);
   this.ShowContextMenu(e, menu);
   return false;
  }
  return showBrowserMenu;
 },
 ShowContextMenu: function(mouseEvent, menu) {
  this.contextMenuActivating = true;
  this.HandleContextMenuHover(menu, mouseEvent);
  menu.ShowInternal(mouseEvent);
 },
 HandleContextMenuHover: function(menu, mouseEvent) {
  menu.handler = menu.handler || this.OnContextMenuCloseUp.bind(this);
  menu.CloseUp.AddHandler(menu.handler);
  if(!this.GetRowHotTrackStyle())
   return;
  this.activeContextMenu = menu;
  this.sourceContextMenuRow = this.grid.getItemByHtmlEvent(mouseEvent);
  ASPx.AddAfterClearHoverState(function(source, args) { this.OnClearHoverState(args.item, args.element, args.toElement); }.bind(this));
 },
 OnClearHoverState: function(hoverItem, hoverElement, newHoverElement) {
  if(!this.activeContextMenu || !this.activeContextMenu.GetVisible() || !this.sourceContextMenuRow) {
   ASPx.RemoveClassNameFromElement(hoverElement, this.GetRowHotTrackStyle()[0]);
   return;
  }
  if(this.sourceContextMenuRow.id === hoverElement.id) {
   var newHoverItem = hoverItem.Clone();
   newHoverItem.Apply(hoverElement);
  }
 },
 OnContextMenuCloseUp: function(e) {
  this.contextMenuActivating = false;
  if(!this.sourceContextMenuRow || !this.activeContextMenu || this.activeContextMenu.GetVisible()) return;
  var stateController = ASPx.GetStateController();
  if(!stateController) return;
  if(stateController.currentHoverElement !== this.sourceContextMenuRow)
   stateController.DoClearHoverState(this.sourceContextMenuRow, null);
  this.sourceContextMenuRow = null;
  this.activeContextMenu = null;
 },
 GetPreparedContextMenu: function(args) { 
  var menu = null;
  switch(args.objectType) {
   case "grouppanel":
    menu = this.GetGroupPanelContextMenu();
    break;
   case "header":
   case "emptyheader":
    menu = this.GetColumnContextMenu();
    break;
   case "row":
   case "grouprow":
   case "emptyrow":
    menu = this.GetRowContextMenu();
    break;
   case "footer":
    menu = this.GetFooterContextMenu();
    break;
   case "groupfooter":
    menu = this.GetGroupFooterContextMenu();
    break;
  }
  if(menu)
   this.ActivateContextMenuItems(menu, args);
  return menu;
 },
 GetContextMenuArgs: function(e) { 
  var args = this.GetContextMenuArgsCore(e);
  if(args.objectType || !this.grid.HasBandedDataRows())
   return args;
  var index = -1;
  var grid = this.grid;
  var dataRowIdPrefix = grid.name + "_" + ASPx.GridViewConsts.DataRowID;
  var isBandedDataRowFound = !!ASPx.GetParent(ASPx.Evt.GetEventSource(e), function(element){
   index = grid.getItemIndex(element.id);
   var level = grid.GetBandedDataRowLevelByID(element.id);
   return element.id == dataRowIdPrefix + index + "_" + level;
  });
  if(isBandedDataRowFound)
   args = { objectType: "row", index: index };
  return args;
 },
 GetContextMenuArgsCore: function(e) { 
  var objectTypes = this.GetObjectTypes();
  var src = ASPx.Evt.GetEventSource(e);
  var element = src;
  while(element && element.tagName !== "BODY") {
   var id = element.id;
   element = element.parentNode;
   if(!id) continue;
   var indexInfo = this.grid.tryGetNumberFromEndOfString(id);
   var index = indexInfo.success ? indexInfo.value : "";
   for(var partialID in objectTypes) {
    if(id == partialID + index) {
     var type = objectTypes[partialID];
     var isGroupFooter = type == "groupfooter";
     if(type == "footer" || isGroupFooter) {
      if(!isGroupFooter)
       index = this.grid.GetFooterCellIndex(src);
      else
       index = this.grid.GetGroupFooterCellIndex(src);
      if(index < 0)
       return { objectType: null, index: -1 };
     } else if(type == "emptyheader" || type == "grouppanel" || type == "emptyrow") {
      index = this.emptyElementIndex;
     }
     return { objectType: type, index: index };
    }
   }
  }
  return { objectType: null, index: -1 };
 },
 ActivateContextMenuItems: function(menu, args) {
  menu.elementInfo = args;
  this.SyncMenuItemsInfoSettings(menu, args.index, menu.cpItemsInfo);
 },
 SyncMenuItemsInfoSettings: function(menu, groupElementIndex, itemsInfo) {
  for(var i = 0; i < menu.GetItemCount() ; ++i) {
   var item = menu.GetItem(i);
   var itemInfo = itemsInfo[item.indexPath];
   var visible = this.GetItemServerState(itemInfo[0], groupElementIndex);
   var enabled = this.GetItemServerState(itemInfo[1], groupElementIndex);
   var checked = this.GetItemServerState(itemInfo[2], groupElementIndex);
   if(item.name === GridViewContextMenuItemType.ShowCustomizationWindow)
    checked = this.IsCustWindowVisible();
   item.SetVisible(visible);
   item.SetEnabled(enabled);
   item.SetChecked(checked);
   if(visible && enabled && !checked)
    this.SyncMenuItemsInfoSettings(item, groupElementIndex, itemsInfo);
  }
 },
 GetContextMenuItemChecked: function(item) {
  var itemInfo = item.menu.cpItemsInfo[item.indexPath];
  var elementIndex = item.menu.elementInfo.index;
  return this.GetItemServerState(itemInfo[2], elementIndex);
 },
 GetItemServerState: function(itemInfo, groupElementIndex) {
  var saveVisible = !!itemInfo[0];
  var indices = itemInfo.length === 1 ? [ ] : itemInfo[1];
  return ASPx.Data.ArrayIndexOf(indices, groupElementIndex) > -1 ? saveVisible : !saveVisible;
 },
 OnContextMenuItemClick: function(e) {
  var elementInfo = e.item.menu.elementInfo;
  this.clickedMenuItem = e.item;
  this.SaveClientState();
  if(this.GetEnableAccessibility() && !e.item.items.length)
   e.item.menu.ProcessLostFocus();
  if(this.RaiseContextMenuItemClick(e, elementInfo))
   return true;
  this.ProcessContextMenuItemClick(e);
  this.ResetState();
 },
 ProcessContextMenuItemClick: function(e){
  var item = e.item;
  var elementInfo = item.menu.elementInfo;
  if(!elementInfo) return;
  var command = this.GetItemCommand(item);
  switch(command) {
   case GridViewContextMenuItemType.FullExpand:
    this.grid.ExpandAll();
    break;
   case GridViewContextMenuItemType.FullCollapse:
    this.grid.CollapseAll();
    break;
   case GridViewContextMenuItemType.SortAscending:
    this.grid.SortBy(elementInfo.index, "ASC", false);
    break;
   case GridViewContextMenuItemType.SortDescending:
    this.grid.SortBy(elementInfo.index, "DSC", false);
    break;
   case GridViewContextMenuItemType.ClearSorting:
    this.grid.SortBy(elementInfo.index, "NONE", false);
    break;
   case GridViewContextMenuItemType.ShowFilterBuilder:
    this.grid.ShowFilterControl();
    break;
   case GridViewContextMenuItemType.ShowFilterRow:
    this.ContextMenuShowFilterRow(!this.GetContextMenuItemChecked(item));
    break;
   case GridViewContextMenuItemType.ClearFilter:
    this.grid.AutoFilterByColumn(this.grid.GetColumn(elementInfo.index));
    break;
   case GridViewContextMenuItemType.ShowFilterRowMenu:
    this.ContextMenuShowFilterRowMenu(!this.GetContextMenuItemChecked(item));
    break;
   case GridViewContextMenuItemType.ShowGroupPanel:
    this.ContextMenuShowGroupPanel(!this.GetContextMenuItemChecked(item));
    break;
   case GridViewContextMenuItemType.ShowSearchPanel:
    this.ContextMenuShowSearchPanel(!this.GetContextMenuItemChecked(item));
    break;
   case GridViewContextMenuItemType.ShowCustomizationDialog:
    this.grid.ShowCustomizationDialog();
    break;
   case GridViewContextMenuItemType.ShowCustomizationWindow:
    if(!this.IsCustWindowVisible())
     this.grid.ShowCustomizationWindow(false, e.item.menu.accessibleFocusElement);
    else
     this.grid.HideCustomizationWindow();
    break;
   case GridViewContextMenuItemType.ShowFooter:
    this.ContextMenuShowFooter(!this.GetContextMenuItemChecked(item));
    break;
   case GridViewContextMenuItemType.ExpandRow:
    this.grid.ExpandRow(elementInfo.index);
    break;
   case GridViewContextMenuItemType.CollapseRow:
    this.grid.CollapseRow(elementInfo.index);
    break;
   case GridViewContextMenuItemType.ExpandDetailRow:
    this.grid.ExpandDetailRow(elementInfo.index);
    break;
   case GridViewContextMenuItemType.CollapseDetailRow:
    this.grid.CollapseDetailRow(elementInfo.index);
    break;
   case GridViewContextMenuItemType.NewRow:
    this.grid.AddNewItem();
    break;
   case GridViewContextMenuItemType.EditRow:
    this.ContextMenuStartEditItem(elementInfo.index, e.item.menu);
    break;
   case GridViewContextMenuItemType.DeleteRow:
    this.grid.DeleteGridItem(elementInfo.index);
    break;
   case GridViewContextMenuItemType.Refresh:
    this.grid.Refresh();
    break;
   case GridViewContextMenuItemType.HideColumn:
    var groupped = ASPx.IsExists(this.grid.GetHeader(elementInfo.index, true));
    this.grid.MoveColumn(elementInfo.index, -1, false, false, groupped);
    break;
   case GridViewContextMenuItemType.ShowColumn:
    this.grid.MoveColumn(elementInfo.index, elementInfo.index);
    break;
   case GridViewContextMenuItemType.SummarySum:
    this.ContextMenuSetSummary(item, elementInfo.index, 0);
    break;
   case GridViewContextMenuItemType.SummaryMin:
    this.ContextMenuSetSummary(item, elementInfo.index, 1);
    break;
   case GridViewContextMenuItemType.SummaryMax:
    this.ContextMenuSetSummary(item, elementInfo.index, 2);
    break;
   case GridViewContextMenuItemType.SummaryCount:
    this.ContextMenuSetSummary(item, elementInfo.index, 3);
    break;
   case GridViewContextMenuItemType.SummaryAverage:
    this.ContextMenuSetSummary(item, elementInfo.index, 4);
    break;
   case GridViewContextMenuItemType.SummaryNone:
    this.ContextMenuClearSummary(elementInfo.index);
    break;
   case GridViewContextMenuItemType.GroupSummarySum:
    this.ContextMenuSetGroupSummary(item, elementInfo.index, 0);
    break;
   case GridViewContextMenuItemType.GroupSummaryMin:
    this.ContextMenuSetGroupSummary(item, elementInfo.index, 1);
    break;
   case GridViewContextMenuItemType.GroupSummaryMax:
    this.ContextMenuSetGroupSummary(item, elementInfo.index, 2);
    break;
   case GridViewContextMenuItemType.GroupSummaryCount:
    this.ContextMenuSetGroupSummary(item, elementInfo.index, 3);
    break;
   case GridViewContextMenuItemType.GroupSummaryAverage:
    this.ContextMenuSetGroupSummary(item, elementInfo.index, 4);
    break;
   case GridViewContextMenuItemType.GroupSummaryNone:
    this.ContextMenuClearGroupSummary(elementInfo);
    break;
   case GridViewContextMenuItemType.ClearGrouping:
    this.ContextMenuClearGrouping();
    break;
   case GridViewContextMenuItemType.GroupByColumn:
    var column = this.grid.GetColumn(elementInfo.index);
    if(column) {
     var order = null;
     if(column.sortOrder !== ASPx.GridSortOrder.None)
      order = column.sortOrder === ASPx.GridSortOrder.Ascending ? "ASC" : "DSC";
     this.grid.GroupBy(elementInfo.index, null, order);
    }
    break;
   case GridViewContextMenuItemType.UngroupColumn:
    this.grid.UnGroup(elementInfo.index);
    break;
  }
  if(this.IsExportItem(command))
   this.ContextMenuExport(item);
 },
 ContextMenuExport: function(item) {
  var command = this.GetItemCommand(item),
   exportFormat = this.GetExportFormat(command);
  this.grid.ExportTo(exportFormat);
 },
 ContextMenuStartEditItem: function(visibleIndex, menu) {
  var rowCells = this.grid.GetRow(visibleIndex).children;
  var colIndex = -1;
  var menuLocationX = menu.GetMainElement().getBoundingClientRect().left;
  for(var i = 0; i < rowCells.length; ++i) {
   var cell = rowCells[i];
   if(cell.tagName !== "TD") continue;
   var cellRect = cell.getBoundingClientRect();
   if(cellRect.left > menuLocationX) break;
   if(menuLocationX > cellRect.left && menuLocationX < cellRect.right) {
    colIndex = i;
    break;
   }
  }
  this.grid.StartEditItem(visibleIndex, colIndex);
 },
 ProcessCustomContextMenuItemClick: function(usePostBack) {
  var args = [ "" ];
  if(usePostBack)
   this.SendContextMenuPostBack(args);
  else
   this.SendContextMenuCallback(args);
 },
 ContextMenuShowGroupPanel: function(show) {
  this.SendContextMenuCallback([ "ShowGroupPanel", show ? 1 : 0 ]);
 },
 ContextMenuShowSearchPanel: function(show) {
  this.SendContextMenuCallback([ "ShowSearchPanel", show ? 1 : 0 ]);
 },
 ContextMenuShowFilterRow: function(show) {
  this.SendContextMenuCallback([ "ShowFilterRow", show ? 1 : 0 ]);
 },
 ContextMenuShowFilterRowMenu: function(show) {
  this.SendContextMenuCallback([ "ShowFilterRowMenu", show ? 1 : 0 ]);
 },
 ContextMenuShowFooter: function(show) {
  this.SendContextMenuCallback([ "ShowFooter", show ? 1 : 0 ]);
 },
 ContextMenuClearGrouping: function() {
  this.SendContextMenuCallback([ "ClearGrouping" ]);
 },
 ContextMenuSetSummary: function(item, index, typeSummary) {
  var checkSummary = this.GetContextMenuItemChecked(item) ? 0 : 1;
  this.SendContextMenuCallback([ "SetSummary", index, typeSummary, checkSummary ]);
 },
 ContextMenuSetGroupSummary: function(item, index, typeSummary, isGroupSummary) {
  var checkSummary = this.GetContextMenuItemChecked(item) ? 0 : 1;
  var isGroupFooterSummary = this.IsGroupFooterMenuItem(item) ? "1" : "0";
  this.SendContextMenuCallback([ "SetGroupSummary", index, typeSummary, checkSummary, isGroupFooterSummary ]);
 },
 ContextMenuClearSummary: function(index) {
  this.SendContextMenuCallback([ "ClearSummary", index ]);
 },
 ContextMenuClearGroupSummary: function(elementInfo) {
  var isGroupFooterSummary = elementInfo.objectType === "groupfooter" ? "1" : "0";
  this.SendContextMenuCallback([ "ClearGroupSummary", elementInfo.index, isGroupFooterSummary ]);
 },
 IsGroupFooterMenuItem: function(item) {
  return item.menu.name === this.grid.GetGroupFooterContextMenuName();
 },
 GetObjectTypes: function() {
  if(!this.objectTypes)
   this.objectTypes = this.CreateObjectTypes();
  return this.objectTypes;
 },
 CreateObjectTypes: function() {
  var gridName = this.GetGridName();
  var result = { };
  result[gridName + "_" + "grouppanel"]          = "grouppanel";
  result[gridName + "_" + ASPx.GridViewConsts.AdaptiveGroupPanelID]   = "grouppanel";
  result[gridName + "_" + ASPx.GridViewConsts.HeaderRowID]   = "emptyheader";
  result[gridName + "_" + "col"]           = "header";
  result[gridName + this.GetCustWindowSuffix() + "_" + "col"]   = "header";
  result[gridName + "_" + "groupcol"]         = "header";
  result[gridName + "_" + ASPx.GridViewConsts.DataRowID]     = "row";
  result[gridName + "_" + ASPx.GridViewConsts.DetailRowID]   = "row";
  result[gridName + "_" + ASPx.GridViewConsts.AdaptiveDetailRowID] = "row";
  result[gridName + "_" + ASPx.GridViewConsts.EmptyDataRowID]   = "emptyrow";
  result[gridName + "_" + ASPx.GridViewConsts.GroupRowID]    = "grouprow";
  result[gridName + "_" + ASPx.GridViewConsts.GroupRowID + "Exp"]  = "grouprow";
  result[gridName + "_" + ASPx.GridViewConsts.FooterRowID]   = "footer";
  result[gridName + "_" + ASPx.GridViewConsts.FilterRowID]   = "filterrow";
  result[gridName + "_" + ASPx.GridViewConsts.GroupFooterRowID]    = "groupfooter";
  return result;
 },
});
ASPx.GridViewContextMenuHelper = GridViewContextMenuHelper;
})();
